#!/usr/bin/perl
#
# lantrip.pl
#
# By Sean Mostafavi (seanm@undersea.net)
#
# --Most of the following is done by lantrip_client.pl--
#
# This needs to be set in sshd_config on each client machine:
# 	RhostsRSAAuthentication yes
#
# The following needs to be executed for each CLIENT:
# 	 	  # su tripwire
#		  $ ssh-keygen -t dsa
#		  $ cp ~/.ssh/id_dsa.pub \
#		  	tripwire@CLIENT:/opt/tripwire/.ssh/authorized_keys2
#
# The random directories must also exist on each client machine!!!
#
# --> Make sure gunzip is in PATH for remote tripwire user! <--
#
# Also, the client machine must have this machine in its known_hosts file
#
# Change the following on the client system's tripwire src include/config.h
#
#	#if !defined(SYSV) || (defined(SYSV) && (SYSV > 2))
#	#define CONFIG_PATH     "data"
#	#define DATABASE_PATH   "data/databases"
#	#else
#	#define CONFIG_PATH     "data"
#	#define DATABASE_PATH   "data/databases"
#	#endif
#	*/
#
#	#define CONFIG_PATH     "data"
#	#define DATABASE_PATH   "data/databases"
# 
# Also make changes to TW Makefile
#
# The remote tripwire user must be using sh

use strict;

# Add your OS_Arcitecture if it is not listed here
my @os_list  		= ('AIX', 'HP-UX', 'Linux_i386', 'Solaris_INTEL', 'Solaris_SPARC',
   			   'Windows', 'FreeBSD', 'NetBSD', 'OpenBSD');
my $l_user   		= 'tripwire';
my $datafile		= 'data_out';
my $config_f 		= 'data/hosts.conf';
my $rand_d_f 		= 'data/rand_dir.conf';
my $tw_dir   		= 'data/r_databases';
my $ssh      		= '/usr/bin/ssh';
my $scp      		= '/usr/bin/scp';
my $ssh_opts 		= '';
my $tw_opts  		= '';
my $usage    		= "$0 [-v] [-i HOST ]\n\n\t-v\tVerbose.\n\t-i HOST\tInit all remote databases.\n";
my $r_data_d 		= 'data';
my $r_db_d 		= "$r_data_d/databases";
my $r_db_d_after	= "databases";
my $db_file 		= 'tw.db_';
my $verbose  		= 0;
my $data_out 		= 0;
my $data_compare	= 0;
my $init_mode 		= 0;
my $h_not_fnd 		= 1;

# Command line options processing.
for (my $i = 0; $i < @ARGV; $i++) {
	if ($ARGV[$i] =~ /^\s*\-v\s*$/) {
		$verbose = 1;
	} elsif ($ARGV[$i] =~ /^\s*-i\s*$/) {
		if ($ARGV[$i+1]) {
			$init_mode = $ARGV[$i+1];	
			chomp($init_mode);
			$init_mode =~ s/^\s*|\s*$//g;
	 		last;
#		} else {
#			die $usage;
		}
	} elsif ($ARGV[$i] =~ /\S+/) {
		die $usage;
	}
}

# Seed rand()
srand();

# Read config files.
my ($hosts, $rand_d) = read_config($config_f, $rand_d_f);

# Cycle through the hosts in the config file.
for (my $i = 0; $i < @$hosts; $i++) {
	my ($host, $user, $os) = split(/ /, $hosts->[$i]);
	
	if ($init_mode) {
		if ($host !~ /$init_mode/i) {
			$h_not_fnd = 0;		
			next;
		}
	}

	my $r_dir = $rand_d->[rand(@$rand_d)];

	# .tar.gz is assumed
	my $final_name = 't';
	
	# Make up some random file names
	print "\nGenerating random files/dirs...\n" if $verbose;
	my $rnd_file = "." . int(rand(50)) . int(rand(50)) . int(rand(50)) . "_libcache.dat";
	my $rnd_dir  = "." . int(rand(50)) . int(rand(50)) . "_tmplib";
	
	print "Copying over Tripwire binary for $os\n\tto $host:$r_dir/$rnd_file\n\n" if $verbose;
	`$scp $tw_dir/$os/tw_$os.tar.gz $user\@$host:$r_dir/$rnd_file`;

	# Run tripwire on remote host
	my $r_hostname = `$ssh $user\@$host hostname`;
	chomp($r_hostname);

	if ($init_mode) {
		print "Initializing Tripwire database on $init_mode...\n" if $verbose;
		print "Working in $host $r_dir/$rnd_dir...\n\n" if $verbose;

		$data_out = `$ssh $user\@$host \"mkdir $r_dir/$rnd_dir ; \
			mv $r_dir/$rnd_file $r_dir/$rnd_dir/$final_name.tar.gz; \
			cd $r_dir/$rnd_dir ; \
			gunzip $final_name.tar.gz ; tar xf $final_name.tar; \
			./tripwire --init;  \
			cd $r_db_d_after; \
			cat $db_file$r_hostname; \	
			cd / ; rm -rf $r_dir/$rnd_dir; \" `;
	
		analyze(\$data_out, $host, $user, $os, "$tw_dir/$os/$datafile-init-$r_hostname.db");
	} else {
		print "Collecting Tripwire data...\n" if $verbose;
		print "Working in $host $r_dir/$rnd_dir...\n\n" if $verbose;

		`$ssh $user\@$host \"mkdir $r_dir/$rnd_dir ; \
			mv $r_dir/$rnd_file $r_dir/$rnd_dir/$final_name.tar.gz; \
			cd $r_dir/$rnd_dir ; \
			gunzip $final_name.tar.gz ; tar xf $final_name.tar; \
			[ -d $r_db_d ] || mkdir $r_db_d; \" `;

		print "Copying over database...\n" if $verbose;
		`$scp $tw_dir/$os/$datafile-init-$r_hostname.db $user\@$host:$r_dir/$rnd_dir/$r_db_d/$db_file$r_hostname`;

		print "Gathering data from comparison...\n" if $verbose;
		$data_compare = `$ssh $user\@$host \"
			cd $r_dir/$rnd_dir ; \
			./tripwire > tmp.file 2>&1; cat tmp.file; \
			cd / ; rm -rf $r_dir/$rnd_dir; \" `;

		analyze(\$data_compare, $host, $user, $os, "$tw_dir/$os/$datafile-comp-$r_hostname.db");
	}
}

if ($init_mode) {
	die "Couldn't find host name $init_mode in $config_f" if ($h_not_fnd);
}

exit;


sub analyze {
	my ($out, $h, $u, $os, $file) = @_;

	open(FILE, "> $file") || 
		die "Couldn't open file $file for writing.\n";

	print FILE $$out;

	close(FILE);
}

sub read_config {
	my ($conf, $rand_d_f) = @_;
	my @hosts  = ();
	my @rand_d = ();
	my $i      = 1;

	open(FILE, $conf) || die "Can't open config file $config_f\n";

	while (<FILE>) {
		if (/^\s*(\S+?\s+\S+?\s+\S+)\s*$/) {
			my ($h, $u, $os) = split(/\s+/, $&);	
			my $os_match     = 0;
			
			$hosts[$i-1] = "$h $u $os";
			
			foreach my $o (@os_list) {
					$os_match = 1 if ($o =~ /$os/i);
			}
			
			unless ($os_match) {
				print "Config file line $i doesn't match any OS listed below:\n\t";
				for (@os_list) { print "$_ "; }
				print "\n";
			}
			
		} else {
			die "Error in config file $config_f line $i\n";
		}
	
		$i++;
	}

	close(FILE);

	open(FILE, $rand_d_f) || die "Can't open random directory file $rand_d_f\n";
	$i = 1;
	
	while (<FILE>) {
		if (/^\s*(\/.+?)\s*$/) {
			$rand_d[$i-1] = "$&";
			$rand_d[$i-1] =~ s/\/$//;
			chomp($rand_d[$i-1]);
		} else {
			die "Error in rand directory file $rand_d_f line $i\n";
		}
	
		$i++;
	}
	
	close(FILE);

	return(\@hosts, \@rand_d);
}

__END__
