#!/usr/bin/perl
#
# lantrip_client.pl
# 
# Configures client host to be used with LANTrip.
#
# By Sean Mostafavi (seanm@undersea.net)

use strict;

my $usage = "$0";
my $tw_src_path;
my $ssh_path;

print "\n\nPlease enter the directory containing\n"; 
print " your Tripwire Acedemic Release source code\n"; 

do {
	print "-> ";
	$tw_src_path = <STDIN>;
	chomp($tw_src_path);
} while ( ! -f "$tw_src_path/include/config.h" );

print "\n\nPlease enter the directory containing\n"; 
print " your OpenSSH config files\n"; 

do {
	print "-> ";
	$ssh_path = <STDIN>;
	chomp($ssh_path);
} while ( ! -f "$ssh_path/sshd_config" );

open(FILE, "$tw_src_path/include/config.h") ||
	die "Couldn't open $tw_src_path/include/config.h.";

my @tw_inc = <FILE>;
close(FILE);

for (my $i = 0; $i < @tw_inc; $i++) {
	if ($tw_inc[$i] =~ /^\s*#\s*define\s+DATABASE_PATH/) {
		$tw_inc[$i] = "#define DATABASE_PATH	\"data/databases\"\n";
	}
	
	if ($tw_inc[$i] =~ /^\s*#\s*define\s+CONFIG_PATH/) {
		$tw_inc[$i] = "#define CONFIG_PATH	\"data\"\n";
	}
}

print "Applying changes to Tripwire config.h file.\n";

open(FILE, "> $tw_src_path/include/config.h") ||
	die "Couldn't open $tw_src_path/include/config.h for writing.";
print FILE @tw_inc;
close(FILE);

open(FILE, "$tw_src_path/Makefile") ||
	die "Couldn't open $tw_src_path/Makefile";
my @tw_mf = <FILE>;
close(FILE);

for (my $i = 0; $i < @tw_mf; $i++) {
	if ($tw_mf[$i] =~ /DESTDIR/) {
		$tw_mf[$i] = "DESTDIR = /opt/tripwire\n";
	}
	
	if ($tw_mf[$i] =~ /DATADIR/) {
		$tw_mf[$i] = "DATADIR = /opt/tripwire/data\n"
	}

	if ($tw_mf[$i] =~ /MANDIR/) {
		$tw_mf[$i] = "MANDIR  = /opt/tripwire/man\n"
	}
}

print "Applying changes to Tripwire Makefile.\n";

open(FILE, "> $tw_src_path/Makefile") ||
	die "Couldn't open $tw_src_path/Makefile for writing.";
print FILE @tw_mf;
close(FILE);

open(FILE, "$ssh_path/sshd_config") ||
	die "Couldn't open $ssh_path/sshd_config";
my @ssh_conf = <FILE>;
close(FILE);

{
	my $rsa = 0;

	for (my $i = 0; $i < @ssh_conf; $i++) {
		if ($ssh_conf[$i] =~ /RhostsRSAAuthentication\s+[Nn][Oo]/) {
			$ssh_conf[$i] = "RhostsRSAAuthentication yes\n";
			$rsa = 1;
		}
		
		if ($ssh_conf[$i] =~ /RhostsRSAAuthentication\s+[Yy][Ee][Ss]/) {
			$rsa = 1;
		}
	}

	if ($rsa == 0) {
		$ssh_conf[@ssh_conf] = "RhostsRSAAuthentication yes\n";
	}
}

print "Applying changes to OpenSSH configuration.\n";

open(FILE, "> $ssh_path/sshd_config") ||
	die "Couldn't open $ssh_path/sshd_config for writing.";
print FILE @ssh_conf;
close(FILE);

print "Done.\n";

print "Building Tripwire ASR in $tw_src_path\n";
`cd $tw_src_path ; make`;

print "Next you will have to compress the Tripwire installation\nand copy it to the central host\n";


exit;
